/****************************************************************************/
/* WebWorks Help API                                                        */
/*                                                                          */
/*                                                                          */
/****************************************************************************/

/*$Id: wwhplat.h,v 1.2 2004/03/11 21:11:18 allums Exp $*/


#ifndef WWHELP_PLATFORM_H
#define WWHELP_PLATFORM_H


#include "wwhelp/wwhcomm.h"


#ifdef __cplusplus
extern "C"
{
#endif

/****************************************************************************/
/* Define NULL                                                              */
/*                                                                          */
/****************************************************************************/

#ifndef NULL
#define NULL 0
#endif


/****************************************************************************/
/* Platform specific WebWorks Help API functions                            */
/*                                                                          */
/****************************************************************************/

int  WWHelp_Platform_Startup();

int  WWHelp_Platform_Shutdown();

int  WWHelp_Platform_SetDefaultBrowserInfo(WWHelp_BrowserInfo* pParamBrowserInfo);

int  WWHelp_Platform_CheckBrowserRunning(const WWHelp_BrowserInfo* pParamBrowserInfo);

int  WWHelp_Platform_LaunchWithURL(const WWHelp_BrowserInfo* pParamBrowserInfo,
                                   const char*               pParamURL,
                                   const size_t              ParamURLLength);

int  WWHelp_Platform_SendURL(const WWHelp_BrowserInfo* pParamBrowserInfo,
                             const char*               pParamURL,
                             const size_t              ParamURLLength);

#ifdef __cplusplus
}
#endif


#endif
